@@ -5,21 +5,18 @@ no warnings 'once'; # guard against -w
 sub _getglob { \*{$_[0]} }
 sub _getstash { \%{"$_[0]::"} }
 
-BEGIN {
-  *lt_5_8_3 = $] < 5.008003
-    ? sub () { 1 }
-    : sub () { 0 }
-  ;
-}
+use constant lt_5_8_3 => ( $] < 5.008003 ) ? 1 : 0;
+use constant can_haz_subname => eval { require Sub::Name };
 
 use strictures 1;
 use Module::Runtime qw(require_module);
+use Devel::GlobalDestruction;
 use base qw(Exporter);
 use Moo::_mro;
 
 our @EXPORT = qw(
     _getglob _install_modifier _load_module _maybe_load_module
-    _get_linear_isa
+    _get_linear_isa _getstash _install_coderef _name_coderef
 );
 
 sub _install_modifier {
@@ -40,7 +37,8 @@ sub _load_module {
   return 1 if $INC{"${proto}.pm"};
   # can't just ->can('can') because a sub-package Foo::Bar::Baz
   # creates a 'Baz::' key in Foo::Bar's symbol table
-  return 1 if grep !/::$/, keys %{_getstash($_[0])||{}};
+  my $stash = _getstash($_[0])||{};
+  return 1 if grep +(!ref($_) and *$_{CODE}), values %$stash;
   require_module($_[0]);
   return 1;
 }
@@ -61,11 +59,16 @@ sub _maybe_load_module {
 }
 
 sub _get_linear_isa {
-    return mro::get_linear_isa($_[0]);
+  return mro::get_linear_isa($_[0]);
 }
 
-our $_in_global_destruction = 0;
-END { $_in_global_destruction = 1 }
+sub _install_coderef {
+  *{_getglob($_[0])} = _name_coderef(@_);
+}
+
+sub _name_coderef {
+  can_haz_subname ? Sub::Name::subname(@_) : $_[1];
+}
 
 sub STANDARD_DESTROY {
   my $self = shift;
@@ -74,7 +77,7 @@ sub STANDARD_DESTROY {
     local $?;
     local $@;
     eval {
-      $self->DEMOLISHALL($_in_global_destruction);
+      $self->DEMOLISHALL(in_global_destruction);
     };
     $@;
   };
